/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package de.hillenbrand.dupfinder;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.swing.*;

public class JDupFinder extends HFrame {
	private static final boolean DEBUG = false;

	// App Prefs
	private static final ApplicationPreferences prefs = new ApplicationPreferences("JDupFinder");
	
	// Variables
	private final DefaultListModel	listModel		= new DefaultListModel();
	private final HFileChooser		fileChooser	= new HFileChooser();
	
	// Menu
	private final JMenuBar 		menubar 			= new JMenuBar();
	private final JMenu 			program			= new JMenu("Program");
	private final JMenuItem 		about				= new JMenuItem("About");
	
	// Widgets
	private final JButton 	startButton 				= new JButton("Start", HIconFactory.getButtonIcon("ok"));
	private final JButton 	quitButton 				= new JButton("Quit", HIconFactory.getButtonIcon("cancel"));
	private final JList		directoryList				= new JList(listModel);
	private final JButton	addDirectoryButton		= new JButton(getIcon("plus.gif"));
	private final JButton	removeDirectoryButton	= new JButton(getIcon("minus.gif"));
	
	public JDupFinder()  {
		super("JDupFinder");
		
		// Properties
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		directoryList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		addDirectoryButton.setMargin(new Insets(1,1,1,1));
		removeDirectoryButton.setMargin(new Insets(1,1,1,1));
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		listModel.addElement(System.getProperty("user.home"));
		
		// Listeners
		this.addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { quit(); }});
		startButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { start(); }});
		quitButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { quit(); }});
		addDirectoryButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { addDirectory(); }});
		removeDirectoryButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { removeDirectory(); }});
		
		// create and set panel
		final JPanel panel = createContentPanel();
		panel.setBorder(UIPreferences.getFrameBorder());
		setContentPane(panel);
		
		// add menus
		createMenuBar();
		setJMenuBar(menubar);
		
		// calculate frame size
		pack();
		JDupFinder.addPrefs("Frame", this);
		JDupFinder.addPrefs("FileChooser", fileChooser);
	}

	void about()  {
		JReqTools.showMessage(this, "JDupFinder 1.1:\n\n(c) 2001-2002 by Markus Hillenbrand\nhttp://www.markus-hillenbran.de\n\nLicense: GPL");
	}

	void addDirectory()  {
		// show dialog
		int retVal = fileChooser.showOpenDialog(this);
		if (retVal != JFileChooser.APPROVE_OPTION) return;
		// add result to list
		try
			{
			String filename = fileChooser.getSelectedFile().getCanonicalPath();
			listModel.addElement(filename);
			directoryList.setSelectedValue(filename, true);
			}
		catch (IOException x) { JReqTools.showException(this, x); }
	}

	static final void addPrefs(String tag, Object value)  {
		prefs.add(tag, value);
	}

	JPanel createContentPanel()  {
		final JPanel panel = new JPanel(new GridBagLayout());
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// Directories
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy=0; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=1;
		panel.add(createDirectoriesPanel(), c);
		
		// Buttons
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
		panel.add(startButton, c);
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=10; c.weighty=0;
		panel.add(new JPanel(), c);
		c.gridx = 2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
		panel.add(quitButton, c);
		
		return panel;
	}

	JPanel createDirectoriesPanel()  {
		final JPanel panel = new JPanel(new GridBagLayout());
		panel.setBorder(BorderFactory.createTitledBorder("Directories to be scanned"));
		
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		c.insets = new Insets(1,1,1,1);
		
		// List
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy=0; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=1;
		panel.add(new JScrollPane(directoryList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);
		
		// Button
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=10; c.weighty=0;
		panel.add(new JPanel(), c);
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		panel.add(removeDirectoryButton, c);
		c.gridx = 2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		panel.add(addDirectoryButton, c);
		
		return panel;
	}

	private void createMenuBar()  {
		// create menu: "Program"
		program.add(about);
		menubar.add(program);
		
		// Listeners
		about.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { about(); } });
	}

	ImageIcon getIcon(String iconName)  {
		try
			{
			InputStream resource = getClass().getResourceAsStream("icons/" + iconName);
			if (resource == null) 
				{
				System.err.println("[ERROR in method "+getClass().getName()+".getIcon(): Can't load icon '" + iconName + "']");
				return null;
				}
			BufferedInputStream in = new BufferedInputStream(resource);
			ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
			byte [] buffer = new byte[1024];
			int n;
			while ((n = in.read(buffer)) > 0) out.write(buffer, 0, n);
			in.close();
			out.flush();
			buffer = out.toByteArray();
			return new ImageIcon(buffer);
			}
		catch (Exception x ) { System.err.println("JDupFinder.getIcon(): " + x); return null; }
	}

	public static void main(String [] args)  {
		// init the ui prefs tool
		UIPreferences.init();
		
		// we should not use localized libraries
		Locale.setDefault(Locale.ENGLISH);
		
		// create and open the main frame
		JDupFinder df = new JDupFinder();
		df.setVisible(true);
	}

	void quit()  {
		prefs.save();
		System.exit(0);
	}

	void removeDirectory()  {
		// get selected object
		String selected = (String) directoryList.getSelectedValue();
		if (selected == null) return;
		// remove it from the model
		listModel.removeElement(selected);
		if (listModel.getSize() > 0) directoryList.setSelectedIndex(0);
	}

	void start()  {
		// check for data
		if (listModel.getSize() == 0)
			{
			JReqTools.showError(this, "You have to add at least one directory to be scanned!");
			return;
			}
		
		// copy data
		final String [] data = new String [listModel.getSize()];
		listModel.copyInto(data);
		
		// open progess tracker
		final JDupFinderProgressTask pTask = new JDupFinderProgressTask(data);
		final ProgressTracker pTracker = new ProgressTracker("JDupFinder: Scanning for Duplicates", "All files are collected and compared to each other:", pTask);
		pTracker.addCancelButton(true);
		pTracker.start(this);
		
		// get the result
		final Vector result = pTask.getDuplicateFiles();
		if (result.size() > 0)
			{
			JDupFinderResultFrame rFrame = new JDupFinderResultFrame(result);
			rFrame.setVisible(true);
			}
		else JReqTools.showMessage(this, "No duplicate files found.");
	}

}
